@@ -5,7 +5,7 @@
  *
  * @author Aleksander Machniak <alec@alec.pl>
  *
- * Copyright (C) 2011-2012, Kolab Systems AG
+ * Copyright (C) Kolab Systems AG
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 
 class acl extends rcube_plugin
 {
-    public $task = 'settings|addressbook|calendar';
+    public $task = 'settings';
 
     private $rc;
     private $supported = null;
@@ -40,9 +40,7 @@ class acl extends rcube_plugin
 
         // Register hooks
         $this->add_hook('folder_form', array($this, 'folder_form'));
-        // kolab_addressbook plugin
-        $this->add_hook('addressbook_form', array($this, 'folder_form'));
-        $this->add_hook('calendar_form_kolab', array($this, 'folder_form'));
+
         // Plugin actions
         $this->register_action('plugin.acl', array($this, 'acl_actions'));
         $this->register_action('plugin.acl-autocomplete', array($this, 'acl_autocomplete'));
@@ -177,7 +175,7 @@ class acl extends rcube_plugin
         $this->rc->output->add_label('save', 'cancel');
         $this->include_script('acl.js');
         $this->rc->output->include_script('list.js');
-        $this->include_stylesheet($this->local_skin_path().'/acl.css');
+        $this->include_stylesheet($this->local_skin_path() . '/acl.css');
 
         // add Info fieldset if it doesn't exist
         if (!isset($args['form']['props']['fieldsets']['info']))
@@ -315,11 +313,15 @@ class acl extends rcube_plugin
     {
         // Create username input
         $attrib['name'] = 'acluser';
+        $class = $attrib['class'];
+        unset($attrib['class']);
 
         $textfield = new html_inputfield($attrib);
 
-        $fields['user'] = html::label(array('for' => $attrib['id']), $this->gettext('username'))
-            . ' ' . $textfield->show();
+        $fields['user'] = html::div('input-group',
+            html::span('input-group-prepend',
+                html::label(array('for' => $attrib['id'], 'class' => 'input-group-text'), $this->gettext('username')))
+            . ' ' . $textfield->show());
 
         // Add special entries
         if (!empty($this->specials)) {
@@ -340,11 +342,11 @@ class acl extends rcube_plugin
                     . $val);
             }
 
-            $out = html::tag('ul', array('id' => 'usertype', 'class' => $attrib['class']), $ul, html::$common_attrib);
+            $out = html::tag('ul', array('id' => 'usertype', 'class' => $class), $ul, html::$common_attrib);
         }
         // Display text input alone
         else {
-            $out = $fields['user'];
+            $out = html::div($class, $fields['user']);
         }
 
         return $out;
@@ -405,10 +407,10 @@ class acl extends rcube_plugin
         }
         else {
             $items = array(
-                'read' => 'lrs',
-                'write' => 'wi',
+                'read'   => 'lrs',
+                'write'  => 'wi',
                 'delete' => $deleteright,
-                'other' => preg_replace('/[lrswi'.$deleteright.']/', '', implode($supported)),
+                'other'  => preg_replace('/[lrswi'.$deleteright.']/', '', implode($supported)),
             );
 
             // give plugins the opportunity to adjust this list
@@ -436,14 +438,19 @@ class acl extends rcube_plugin
 
             // filter out virtual rights (c or d) the server may return
             $userrights = array_intersect($rights, $supported);
-            $userid = rcube_utils::html_identifier($user);
+            $userid     = rcube_utils::html_identifier($user);
+            $title      = null;
 
             if (!empty($this->specials) && in_array($user, $this->specials)) {
-                $user = $this->gettext($user);
+                $username = $this->gettext($user);
+            }
+            else {
+                $username = $this->resolve_acl_identifier($user, $title);
             }
 
-            $table->add_row(array('id' => 'rcmrow'.$userid));
-            $table->add('user', html::a(array('id' => 'rcmlinkrow'.$userid), rcube::Q($user)));
+            $table->add_row(array('id' => 'rcmrow' . $userid, 'data-userid' => $user));
+            $table->add(array('class' => 'user text-nowrap', 'title' => $title),
+                html::a(array('id' => 'rcmlinkrow' . $userid), rcube::Q($username)));
 
             foreach ($items as $key => $right) {
                 $in = $this->acl_compare($userrights, $right);
@@ -452,7 +459,7 @@ class acl extends rcube_plugin
                     case 1: $class = 'partial'; break;
                     default: $class = 'disabled'; break;
                 }
-                $table->add('acl' . $key . ' ' . $class, '');
+                $table->add('acl' . $key . ' ' . $class, '<span></span>');
             }
 
             $js_table[$userid] = implode($userrights);
@@ -513,9 +520,15 @@ class acl extends rcube_plugin
 
             if ($user != $_SESSION['username'] && $username != $_SESSION['username']) {
                 if ($this->rc->storage->set_acl($mbox, $user, $acl)) {
-                    $ret = array('id' => rcube_utils::html_identifier($user),
-                         'username' => $username, 'acl' => implode($acl), 'old' => $oldid);
-                    $this->rc->output->command('acl_update', $ret);
+                    $display = $this->resolve_acl_identifier($username, $title);
+                    $this->rc->output->command('acl_update', array(
+                            'id'       => rcube_utils::html_identifier($user),
+                            'username' => $username,
+                            'title'    => $title,
+                            'display'  => $display,
+                            'acl'      => implode($acl),
+                            'old'      => $oldid
+                    ));
                     $result++;
                 }
             }
@@ -606,8 +619,9 @@ class acl extends rcube_plugin
             }
         }
 
-        if (count($list) == count($supported))
+        if (count($list) == count($supported)) {
             return rcube::Q($this->gettext('aclfull'));
+        }
 
         return html::tag('ul', $attrib, implode("\n", $list));
     }
@@ -634,12 +648,15 @@ class acl extends rcube_plugin
         $cnt1 = count($res);
         $cnt2 = count($acl2);
 
-        if ($cnt1 == $cnt2)
+        if ($cnt1 == $cnt2) {
             return 2;
-        else if ($cnt1)
+        }
+
+        if ($cnt1) {
             return 1;
-        else
-            return 0;
+        }
+
+        return 0;
     }
 
     /**
@@ -785,4 +802,51 @@ class acl extends rcube_plugin
 
         return $user;
     }
+
+    /**
+     * Resolve acl identifier to user/group name
+     */
+    protected function resolve_acl_identifier($id, &$title = null)
+    {
+        if ($this->init_ldap()) {
+            $groups      = $this->rc->config->get('acl_groups');
+            $prefix      = $this->rc->config->get('acl_group_prefix');
+            $group_field = $this->rc->config->get('acl_group_field', 'name');
+
+            // Unfortunately this works only if group_field=name,
+            // list_groups() allows searching by group name only
+            if ($groups && $prefix && $group_field === 'name' && strpos($id, $prefix) === 0) {
+                $gid    = substr($id, strlen($prefix));
+                $result = $this->ldap->list_groups($gid, rcube_addressbook::SEARCH_STRICT);
+
+                if (count($result) === 1 && ($record = $result[0])) {
+                    if ($record[$group_field] === $gid) {
+                        $display = $record['name'];
+                        if ($display != $gid) {
+                            $title = sprintf('%s (%s)', $display, $gid);
+                        }
+
+                        return $display;
+                    }
+                }
+
+                return $id;
+            }
+
+            $this->ldap->set_pagesize('2');
+            // Note: 'uid' works here because we overwrite fieldmap in init_ldap() above
+            $result = $this->ldap->search('uid', $id, rcube_addressbook::SEARCH_STRICT);
+
+            if ($result->count === 1 && ($record = $result->first())) {
+                if ($record['uid'] === $id) {
+                    $title   = rcube_addressbook::compose_search_name($record);
+                    $display = rcube_addressbook::compose_list_name($record);
+
+                    return $display;
+                }
+            }
+        }
+
+        return $id;
+    }
 }
