@@ -6,7 +6,7 @@
  * Plugin that adds a new button to the mailbox toolbar
  * to move messages to a (user selectable) archive folder.
  *
- * @version 3.0
+ * @version 3.2
  * @license GNU GPLv3+
  * @author Andre Rodier, Thomas Bruederli, Aleksander Machniak
  */
@@ -14,7 +14,13 @@ class archive extends rcube_plugin
 {
     public $task = 'settings|mail|login';
 
+    private $archive_folder;
+    private $folders;
 
+
+    /**
+     * Plugin initialization.
+     */
     function init()
     {
         $rcmail = rcmail::get_instance();
@@ -22,14 +28,10 @@ class archive extends rcube_plugin
         // register special folder type
         rcube_storage::$folder_types[] = 'archive';
 
-        if ($rcmail->task == 'mail' && ($rcmail->action == '' || $rcmail->action == 'show')
-            && ($archive_folder = $rcmail->config->get('archive_mbox'))
-        ) {
-            $skin_path = $this->local_skin_path();
-            if (is_file($this->home . "/$skin_path/archive.css")) {
-                $this->include_stylesheet("$skin_path/archive.css");
-            }
+        $this->archive_folder = $rcmail->config->get('archive_mbox');
 
+        if ($rcmail->task == 'mail' && ($rcmail->action == '' || $rcmail->action == 'show') && $this->archive_folder) {
+            $this->include_stylesheet($this->local_skin_path() . '/archive.css');
             $this->include_script('archive.js');
             $this->add_texts('localization', true);
             $this->add_button(
@@ -43,6 +45,7 @@ class archive extends rcube_plugin
                     'height'   => 32,
                     'title'    => 'buttontitle',
                     'domain'   => $this->ID,
+                    'innerclass' => 'inner',
                 ),
                 'toolbar');
 
@@ -50,7 +53,7 @@ class archive extends rcube_plugin
             $this->add_hook('render_mailboxlist', array($this, 'render_mailboxlist'));
 
             // set env variables for client
-            $rcmail->output->set_env('archive_folder', $archive_folder);
+            $rcmail->output->set_env('archive_folder', $this->archive_folder);
             $rcmail->output->set_env('archive_type', $rcmail->config->get('archive_type',''));
         }
         else if ($rcmail->task == 'mail') {
@@ -60,6 +63,13 @@ class archive extends rcube_plugin
         else if ($rcmail->task == 'settings') {
             $this->add_hook('preferences_list', array($this, 'prefs_table'));
             $this->add_hook('preferences_save', array($this, 'save_prefs'));
+
+            if ($rcmail->action == 'folders' && $this->archive_folder) {
+                $this->include_stylesheet($this->local_skin_path() . '/archive.css');
+                $this->include_script('archive.js');
+                // set env variables for client
+                $rcmail->output->set_env('archive_folder', $this->archive_folder);
+            }
         }
     }
 
@@ -68,18 +78,14 @@ class archive extends rcube_plugin
      */
     function render_mailboxlist($p)
     {
-        $rcmail         = rcmail::get_instance();
-        $archive_folder = $rcmail->config->get('archive_mbox');
-        $show_real_name = $rcmail->config->get('show_real_foldernames');
-
         // set localized name for the configured archive folder
-        if ($archive_folder && !$show_real_name) {
-            if (isset($p['list'][$archive_folder])) {
-                $p['list'][$archive_folder]['name'] = $this->gettext('archivefolder');
+        if ($this->archive_folder && !rcmail::get_instance()->config->get('show_real_foldernames')) {
+            if (isset($p['list'][$this->archive_folder])) {
+                $p['list'][$this->archive_folder]['name'] = $this->gettext('archivefolder');
             }
             else {
                 // search in subfolders
-                $this->_mod_folder_name($p['list'], $archive_folder, $this->gettext('archivefolder'));
+                $this->_mod_folder_name($p['list'], $this->archive_folder, $this->gettext('archivefolder'));
             }
         }
 
@@ -125,15 +131,13 @@ class archive extends rcube_plugin
         $delimiter      = $storage->get_hierarchy_delimiter();
         $read_on_move   = (bool) $rcmail->config->get('read_on_archive');
         $archive_type   = $rcmail->config->get('archive_type', '');
-        $archive_folder = $rcmail->config->get('archive_mbox');
-        $archive_prefix = $archive_folder . $delimiter;
+        $archive_prefix = $this->archive_folder . $delimiter;
         $search_request = rcube_utils::get_input_value('_search', rcube_utils::INPUT_GPC);
 
         // count messages before changing anything
         if ($_POST['_from'] != 'show') {
             $threading = (bool) $storage->get_threading();
             $old_count = $storage->count(null, $threading ? 'THREADS' : 'ALL');
-            $old_pages = ceil($old_count / $storage->get_pagesize());
         }
 
         $count = 0;
@@ -148,21 +152,21 @@ class archive extends rcube_plugin
         );
 
         foreach (rcmail::get_uids(null, null, $multifolder, rcube_utils::INPUT_POST) as $mbox => $uids) {
-            if (!$archive_folder || strpos($mbox, $archive_prefix) === 0) {
+            if (!$this->archive_folder || $mbox === $this->archive_folder || strpos($mbox, $archive_prefix) === 0) {
                 $count = count($uids);
                 continue;
             }
             else if (!$archive_type || $archive_type == 'folder') {
-                $folder = $archive_folder;
+                $folder = $this->archive_folder;
 
                 if ($archive_type == 'folder') {
                     // compose full folder path
                     $folder .= $delimiter . $mbox;
-
-                    // create archive subfolder if it doesn't yet exist
-                    $this->subfolder_worker($folder);
                 }
 
+                // create archive subfolder if it doesn't yet exist
+                $this->subfolder_worker($folder);
+
                 $count += $this->move_messages_worker($uids, $mbox, $folder, $read_on_move);
             }
             else {
@@ -186,21 +190,19 @@ class archive extends rcube_plugin
                             . $delimiter . $rcmail->format_date($message->timestamp, 'm');
                         break;
 
-                    case 'sender':
-                        $from = $message->get('from');
-                        preg_match('/[\b<](.+@.+)[\b>]/i', $from, $m);
-                        $subfolder = $m[1] ?: $this->gettext('unkownsender');
+                    case 'tbmonth':
+                        $subfolder = $rcmail->format_date($message->timestamp, 'Y')
+                            . $delimiter . $rcmail->format_date($message->timestamp, 'Y')
+                            . '-' . $rcmail->format_date($message->timestamp, 'm');
+                        break;
 
-                        // replace reserved characters in folder name
-                        $repl = $delimiter == '-' ? '_' : '-';
-                        $replacements[$delimiter] = $repl;
-                        $replacements['.'] = $repl;  // some IMAP server do not allow . characters
-                        $subfolder = strtr($subfolder, $replacements);
+                    case 'sender':
+                        $subfolder = $this->sender_subfolder($message->get('from'));
                         break;
                     }
 
                     // compose full folder path
-                    $folder = $archive_folder . ($subfolder ? $delimiter . $subfolder : '');
+                    $folder = $this->archive_folder . ($subfolder ? $delimiter . $subfolder : '');
 
                     $execute[$folder][] = $message->uid;
                 }
@@ -255,6 +257,7 @@ class archive extends rcube_plugin
         $pages          = ceil($msg_count / $page_size);
         $nextpage_count = $old_count - $page_size * $page;
         $remaining      = $msg_count - $page_size * ($page - 1);
+        $quota_root     = $multifolder ? $this->result['sources'][0] : 'INBOX';
 
         // jump back one page (user removed the whole last page)
         if ($page > 1 && $remaining == 0) {
@@ -264,22 +267,16 @@ class archive extends rcube_plugin
             $jump_back = true;
         }
 
+        // update unread messages counts for all involved folders
+        foreach ($this->result['sources'] as $folder) {
+            rcmail_send_unread_count($folder, true);
+        }
+
         // update message count display
         $rcmail->output->set_env('messagecount', $msg_count);
         $rcmail->output->set_env('current_page', $page);
         $rcmail->output->set_env('pagecount', $pages);
         $rcmail->output->set_env('exists', $exists);
-
-        // update mailboxlist
-        $unseen_count = $msg_count ? $storage->count($mbox, 'UNSEEN') : 0;
-        $old_unseen   = rcmail_get_unseen_count($mbox);
-        $quota_root   = $multifolder ? $this->result['sources'][0] : 'INBOX';
-
-        if ($old_unseen != $unseen_count) {
-            $rcmail->output->command('set_unread_count', $mbox, $unseen_count, ($mbox == 'INBOX'));
-            rcmail_set_unseen_count($mbox, $unseen_count);
-        }
-
         $rcmail->output->command('set_quota', $rcmail->quota_content(null, $quota_root));
         $rcmail->output->command('set_rowcount', rcmail_get_messagecount_text($msg_count), $mbox);
 
@@ -351,7 +348,7 @@ class archive extends rcube_plugin
         $delimiter = $storage->get_hierarchy_delimiter();
 
         if ($this->folders === null) {
-            $this->folders = $storage->list_folders('', $archive_folder . '*', 'mail', null, true);
+            $this->folders = $storage->list_folders('', $this->archive_folder . '*', 'mail', null, true);
         }
 
         if (!in_array($folder, $this->folders)) {
@@ -402,31 +399,36 @@ class archive extends rcube_plugin
             }
 
             $args['blocks']['main']['options']['archive_mbox'] = array(
-                'title'   => $this->gettext('archivefolder'),
-                'content' => $select->show($mbox, array('name' => "_archive_mbox"))
+                'title'   => html::label('_archive_mbox', rcube::Q($this->gettext('archivefolder'))),
+                'content' => $select->show($mbox, array('id' => '_archive_mbox', 'name' => '_archive_mbox'))
             );
 
-            // add option for structuring the archive folder
-            $archive_type = new html_select(array('name' => '_archive_type', 'id' => 'ff_archive_type'));
-            $archive_type->add($this->gettext('none'), '');
-            $archive_type->add($this->gettext('archivetypeyear'), 'year');
-            $archive_type->add($this->gettext('archivetypemonth'), 'month');
-            $archive_type->add($this->gettext('archivetypesender'), 'sender');
-            $archive_type->add($this->gettext('archivetypefolder'), 'folder');
+            // If the server supports only either messages or folders in a folder
+            // we do not allow archive splitting, for simplicity (#5057)
+            if ($rcmail->get_storage()->get_capability(rcube_storage::DUAL_USE_FOLDERS)) {
+                // add option for structuring the archive folder
+                $archive_type = new html_select(array('name' => '_archive_type', 'id' => 'ff_archive_type'));
+                $archive_type->add($this->gettext('none'), '');
+                $archive_type->add($this->gettext('archivetypeyear'), 'year');
+                $archive_type->add($this->gettext('archivetypemonth'), 'month');
+                $archive_type->add($this->gettext('archivetypetbmonth'), 'tbmonth');
+                $archive_type->add($this->gettext('archivetypesender'), 'sender');
+                $archive_type->add($this->gettext('archivetypefolder'), 'folder');
 
-            $args['blocks']['archive'] = array(
-                'name' => rcube::Q($this->gettext('settingstitle')),
-                'options' => array('archive_type' => array(
-                        'title'   => $this->gettext('archivetype'),
-                        'content' => $archive_type->show($type)
+                $args['blocks']['archive'] = array(
+                    'name'    => rcube::Q($this->gettext('settingstitle')),
+                    'options' => array('archive_type' => array(
+                            'title'   => html::label('ff_archive_type', rcube::Q($this->gettext('archivetype'))),
+                            'content' => $archive_type->show($type)
+                        )
                     )
-                )
-            );
+                );
+            }
         }
         else if ($args['section'] == 'server' && !in_array('read_on_archive', $dont_override)) {
             $chbox = new html_checkbox(array('name' => '_read_on_archive', 'id' => 'ff_read_on_archive', 'value' => 1));
             $args['blocks']['main']['options']['read_on_archive'] = array(
-                'title'   => $this->gettext('readonarchive'),
+                'title'   => html::label('ff_read_on_archive', rcube::Q($this->gettext('readonarchive'))),
                 'content' => $chbox->show($rcmail->config->get('read_on_archive') ? 1 : 0)
             );
         }
@@ -451,4 +453,53 @@ class archive extends rcube_plugin
 
         return $args;
     }
+
+    /**
+     * Create folder name from the message sender address
+     */
+    protected function sender_subfolder($from)
+    {
+        static $delim;
+        static $vendor;
+        static $skip_hidden;
+
+        preg_match('/[\b<](.+@.+)[\b>]/i', $from, $m);
+
+        if (empty($m[1])) {
+            return $this->gettext('unkownsender');
+        }
+
+        if ($delim === null) {
+            $rcmail      = rcmail::get_instance();
+            $storage     = $rcmail->get_storage();
+            $delim       = $storage->get_hierarchy_delimiter();
+            $vendor      = $storage->get_vendor();
+            $skip_hidden = $rcmail->config->get('imap_skip_hidden_folders');
+        }
+
+        // Remove some forbidden characters
+        $regexp = '\\x00-\\x1F\\x7F%*';
+
+        if ($vendor == 'cyrus') {
+            // List based on testing Kolab's Cyrus-IMAP 2.5
+            $regexp .= '!`(){}|\\?<;"';
+        }
+
+        $folder_name = preg_replace("/[$regexp]/", '', $m[1]);
+
+        if ($skip_hidden && $folder_name[0] == '.') {
+            $folder_name = substr($folder_name, 1);
+        }
+
+        $replace = $delim == '-' ? '_' : '-';
+        $replacements[$delim] = $replace;
+
+        // Cyrus-IMAP does not allow @ character in folder name
+        if ($vendor == 'cyrus') {
+            $replacements['@'] = $replace;
+        }
+
+        // replace reserved characters in folder name
+        return strtr($folder_name, $replacements);
+    }
 }
