@@ -1,10 +1,11 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2014, The Roundcube Dev Team                       |
- | Copyright (C) 2011, Kolab Systems AG                                  |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
+ | Copyright (C) Kolab Systems AG                                        |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -15,37 +16,36 @@
  +-----------------------------------------------------------------------+
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  | Author: Aleksander Machniak <alec@alec.pl>                            |
+ | Author: Cor Bosman <cor@roundcu.be>                                   |
  +-----------------------------------------------------------------------+
 */
 
 /**
- * Class to provide database supported session storage
+ * Abstract class to provide database supported session storage
  *
  * @package    Framework
  * @subpackage Core
- * @author     Thomas Bruederli <roundcube@gmail.com>
- * @author     Aleksander Machniak <alec@alec.pl>
  */
-class rcube_session
+abstract class rcube_session
 {
-    private $db;
-    private $ip;
-    private $start;
-    private $changed;
-    private $time_diff = 0;
-    private $reloaded = false;
-    private $appends = array();
-    private $unsets = array();
-    private $gc_handlers = array();
-    private $cookiename = 'roundcube_sessauth';
-    private $vars;
-    private $key;
-    private $now;
-    private $secret = '';
-    private $ip_check = false;
-    private $logging = false;
-    private $storage;
-    private $memcache;
+    protected $config;
+    protected $key;
+    protected $ip;
+    protected $changed;
+    protected $start;
+    protected $vars;
+    protected $now;
+    protected $time_diff    = 0;
+    protected $reloaded     = false;
+    protected $appends      = array();
+    protected $unsets       = array();
+    protected $gc_enabled   = 0;
+    protected $gc_handlers  = array();
+    protected $cookiename   = 'roundcube_sessauth';
+    protected $ip_check     = false;
+    protected $logging      = false;
+    protected $ignore_write = false;
+
 
     /**
      * Blocks session data from being written to database.
@@ -54,118 +54,130 @@ class rcube_session
      */
     public $nowrite = false;
 
+    /**
+     * Factory, returns driver-specific instance of the class
+     *
+     * @param object $config
+     * @return Object rcube_session
+     */
+    public static function factory($config)
+    {
+        // get session storage driver
+        $storage = $config->get('session_storage', 'db');
+
+        // class name for this storage
+        $class = "rcube_session_" . $storage;
+
+        // try to instantiate class
+        if (class_exists($class)) {
+            return new $class($config);
+        }
+
+        // no storage found, raise error
+        rcube::raise_error(array('code' => 604, 'type' => 'session',
+                               'line' => __LINE__, 'file' => __FILE__,
+                               'message' => "Failed to find session driver. Check session_storage config option"),
+                           true, true);
+    }
 
     /**
-     * Default constructor
+     * Object constructor
+     *
+     * @param Object $config
      */
-    public function __construct($db, $config)
+    public function __construct($config)
     {
-        $this->db      = $db;
-        $this->start   = microtime(true);
-        $this->ip      = rcube_utils::remote_addr();
-        $this->logging = $config->get('log_session', false);
+        $this->config = $config;
 
-        $lifetime = $config->get('session_lifetime', 1) * 60;
-        $this->set_lifetime($lifetime);
+        // set ip check
+        $this->set_ip_check($this->config->get('ip_check'));
 
-        // use memcache backend
-        $this->storage = $config->get('session_storage', 'db');
-        if ($this->storage == 'memcache') {
-            $this->memcache = rcube::get_instance()->get_memcache();
-
-            // set custom functions for PHP session management if memcache is available
-            if ($this->memcache) {
-                ini_set('session.serialize_handler', 'php');
-
-                session_set_save_handler(
-                    array($this, 'open'),
-                    array($this, 'close'),
-                    array($this, 'mc_read'),
-                    array($this, 'mc_write'),
-                    array($this, 'mc_destroy'),
-                    array($this, 'gc'));
-            }
-            else {
-                rcube::raise_error(array('code' => 604, 'type' => 'db',
-                    'line' => __LINE__, 'file' => __FILE__,
-                    'message' => "Failed to connect to memcached. Please check configuration"),
-                true, true);
-            }
-        }
-        else if ($this->storage != 'php') {
-            ini_set('session.serialize_handler', 'php');
-
-            // set custom functions for PHP session management
-            session_set_save_handler(
-                array($this, 'open'),
-                array($this, 'close'),
-                array($this, 'db_read'),
-                array($this, 'db_write'),
-                array($this, 'db_destroy'),
-                array($this, 'gc'));
-
-            $this->table_name = $this->db->table_name('session', true);
+        // set cookie name
+        if ($name = $this->config->get('session_auth_name')) {
+            $this->set_cookiename($name);
         }
     }
 
+    /**
+     * Register session handler
+     */
+    public function register_session_handler()
+    {
+        if (session_id()) {
+            // Session already exists, skip
+            return;
+        }
+
+        ini_set('session.serialize_handler', 'php');
+
+        // set custom functions for PHP session management
+        session_set_save_handler(
+            array($this, 'open'),
+            array($this, 'close'),
+            array($this, 'read'),
+            array($this, 'sess_write'),
+            array($this, 'destroy'),
+            array($this, 'gc')
+        );
+    }
 
     /**
      * Wrapper for session_start()
      */
     public function start()
     {
+        $this->start   = microtime(true);
+        $this->ip      = rcube_utils::remote_addr();
+        $this->logging = $this->config->get('session_debug', false);
+
+        $lifetime = $this->config->get('session_lifetime', 1) * 60;
+        $this->set_lifetime($lifetime);
+
         session_start();
-
-        // copy some session properties to object vars
-        if ($this->storage == 'php') {
-            $this->key     = session_id();
-            $this->ip      = $_SESSION['__IP'];
-            $this->changed = $_SESSION['__MTIME'];
-        }
     }
 
-
-    public function open($save_path, $session_name)
-    {
-        return true;
-    }
-
-
-    public function close()
-    {
-        return true;
-    }
-
-
     /**
-     * Delete session data for the given key
-     *
-     * @param string Session ID
+     * Abstract methods should be implemented by driver classes
      */
-    public function destroy($key)
+    abstract function open($save_path, $session_name);
+    abstract function close();
+    abstract function destroy($key);
+    abstract function read($key);
+    abstract function write($key, $vars);
+    abstract function update($key, $newvars, $oldvars);
+
+    /**
+     * Session write handler. This calls the implementation methods for write/update after some initial checks.
+     *
+     * @param string $key  Session identifier
+     * @param string $vars Serialized data string
+     *
+     * @return bool True on success, False on failure
+     */
+    public function sess_write($key, $vars)
     {
-        if ($this->storage == 'memcache' && $this->memcache) {
-            return $this->mc_destroy($key);
-        }
-        else if ($this->storage == 'db') {
-            return $this->db_destroy($key);
-        }
-        else {
+        if ($this->nowrite) {
             return true;
         }
-    }
 
+        // check cache
+        $oldvars = $this->get_cache($key);
+
+        // if there are cached vars, update store, else insert new data
+        if ($oldvars) {
+            $newvars = $this->_fixvars($vars, $oldvars);
+            return $this->update($key, $newvars, $oldvars);
+        }
+        else {
+            return $this->write($key, $vars);
+        }
+    }
 
     /**
      * Wrapper for session_write_close()
      */
     public function write_close()
     {
-        if ($this->storage == 'php') {
-            $_SESSION['__IP'] = $this->ip;
-            $_SESSION['__MTIME'] = time();
-        }
-
         session_write_close();
 
         // write_close() is called on script shutdown, see rcube::shutdown()
@@ -174,91 +186,28 @@ class rcube_session
         $this->gc_shutdown();
     }
 
-
     /**
-     * Read session data from database
+     * Creates a new (separate) session
      *
-     * @param string Session ID
+     * @param array $data Session data
      *
-     * @return string Session vars
+     * @return string Session identifier (on success)
      */
-    public function db_read($key)
+    public function create($data)
     {
-        $sql_result = $this->db->query(
-            "SELECT `vars`, `ip`, `changed`, " . $this->db->now() . " AS ts"
-            . " FROM {$this->table_name} WHERE `sess_id` = ?", $key);
+        $length = strlen(session_id());
+        $key    = rcube_utils::random_bytes($length);
 
-        if ($sql_result && ($sql_arr = $this->db->fetch_assoc($sql_result))) {
-            $this->time_diff = time() - strtotime($sql_arr['ts']);
-            $this->changed   = strtotime($sql_arr['changed']);
-            $this->ip        = $sql_arr['ip'];
-            $this->vars      = base64_decode($sql_arr['vars']);
-            $this->key       = $key;
-
-            return !empty($this->vars) ? (string) $this->vars : '';
+        // create new session
+        if ($this->write($key, $this->serialize($data))) {
+            return $key;
         }
-
-        return null;
     }
 
-
-    /**
-     * Save session data.
-     * handler for session_read()
-     *
-     * @param string Session ID
-     * @param string Serialized session vars
-     *
-     * @return boolean True on success
-     */
-    public function db_write($key, $vars)
-    {
-        $now = $this->db->now();
-        $ts  = microtime(true);
-
-        if ($this->nowrite)
-            return true;
-
-        // no session row in DB (db_read() returns false)
-        if (!$this->key) {
-            $oldvars = null;
-        }
-        // use internal data from read() for fast requests (up to 0.5 sec.)
-        else if ($key == $this->key && (!$this->vars || $ts - $this->start < 0.5)) {
-            $oldvars = $this->vars;
-        }
-        else { // else read data again from DB
-            $oldvars = $this->db_read($key);
-        }
-
-        if ($oldvars !== null) {
-            $newvars = $this->_fixvars($vars, $oldvars);
-
-            if ($newvars !== $oldvars) {
-                $this->db->query("UPDATE {$this->table_name} "
-                    . "SET `changed` = $now, `vars` = ? WHERE `sess_id` = ?",
-                    base64_encode($newvars), $key);
-            }
-            else if ($ts - $this->changed + $this->time_diff > $this->lifetime / 2) {
-                $this->db->query("UPDATE {$this->table_name} SET `changed` = $now"
-                    . " WHERE `sess_id` = ?", $key);
-            }
-        }
-        else {
-            $this->db->query("INSERT INTO {$this->table_name}"
-                . " (`sess_id`, `vars`, `ip`, `created`, `changed`)"
-                . " VALUES (?, ?, ?, $now, $now)",
-                $key, base64_encode($vars), (string)$this->ip);
-        }
-
-        return true;
-    }
-
-
     /**
      * Merge vars with old vars and apply unsets
      */
-    private function _fixvars($vars, $oldvars)
+    protected function _fixvars($vars, $oldvars)
     {
         if ($oldvars !== null) {
             $a_oldvars = $this->unserialize($oldvars);
@@ -288,107 +237,21 @@ class rcube_session
         return $newvars;
     }
 
-
-    /**
-     * Handler for session_destroy()
-     *
-     * @param string Session ID
-     *
-     * @return boolean True on success
-     */
-    public function db_destroy($key)
-    {
-        if ($key) {
-            $this->db->query("DELETE FROM {$this->table_name} WHERE `sess_id` = ?", $key);
-        }
-
-        return true;
-    }
-
-
-    /**
-     * Read session data from memcache
-     *
-     * @param string Session ID
-     * @return string Session vars
-     */
-    public function mc_read($key)
-    {
-        if ($value = $this->memcache->get($key)) {
-            $arr = unserialize($value);
-            $this->changed = $arr['changed'];
-            $this->ip      = $arr['ip'];
-            $this->vars    = $arr['vars'];
-            $this->key     = $key;
-
-            return !empty($this->vars) ? (string) $this->vars : '';
-        }
-
-        return null;
-    }
-
-
-    /**
-     * Save session data.
-     * handler for session_read()
-     *
-     * @param string Session ID
-     * @param string Serialized session vars
-     *
-     * @return boolean True on success
-     */
-    public function mc_write($key, $vars)
-    {
-        $ts = microtime(true);
-
-        // no session data in cache (mc_read() returns false)
-        if (!$this->key)
-            $oldvars = null;
-        // use internal data for fast requests (up to 0.5 sec.)
-        else if ($key == $this->key && (!$this->vars || $ts - $this->start < 0.5))
-            $oldvars = $this->vars;
-        else // else read data again
-            $oldvars = $this->mc_read($key);
-
-        $newvars = $oldvars !== null ? $this->_fixvars($vars, $oldvars) : $vars;
-
-        if ($newvars !== $oldvars || $ts - $this->changed > $this->lifetime / 3) {
-            return $this->memcache->set($key, serialize(array('changed' => time(), 'ip' => $this->ip, 'vars' => $newvars)),
-                MEMCACHE_COMPRESSED, $this->lifetime + 60);
-        }
-
-        return true;
-    }
-
-
-    /**
-     * Handler for session_destroy() with memcache backend
-     *
-     * @param string Session ID
-     *
-     * @return boolean True on success
-     */
-    public function mc_destroy($key)
-    {
-        if ($key) {
-            // #1488592: use 2nd argument
-            $this->memcache->delete($key, 0);
-        }
-
-        return true;
-    }
-
-
     /**
      * Execute registered garbage collector routines
+     *
+     * @param int $maxlifetime Maximum session lifetime
+     *
+     * @return bool True on success, False on failure
      */
     public function gc($maxlifetime)
     {
         // move gc execution to the script shutdown function
         // see rcube::shutdown() and rcube_session::write_close()
-        return $this->gc_enabled = $maxlifetime;
-    }
+        $this->gc_enabled = $maxlifetime;
 
+        return true;
+    }
 
     /**
      * Register additional garbage collector functions
@@ -406,48 +269,75 @@ class rcube_session
         $this->gc_handlers[] = $func;
     }
 
-
     /**
      * Garbage collector handler to run on script shutdown
      */
     protected function gc_shutdown()
     {
         if ($this->gc_enabled) {
-            // just delete all expired sessions
-            if ($this->storage == 'db') {
-                $this->db->query("DELETE FROM {$this->table_name}"
-                    . " WHERE `changed` < " . $this->db->now(-$this->gc_enabled));
-            }
-
             foreach ($this->gc_handlers as $fct) {
                 call_user_func($fct);
             }
         }
     }
 
-
     /**
      * Generate and set new session id
      *
      * @param boolean $destroy If enabled the current session will be destroyed
+     *
+     * @return bool True on success, False on failure
      */
-    public function regenerate_id($destroy=true)
+    public function regenerate_id($destroy = true)
     {
+        $old_id = session_id();
+
+        // Since PHP 7.0 session_regenerate_id() will cause the old
+        // session data update, we don't need this
+        $this->ignore_write = true;
         session_regenerate_id($destroy);
+        $this->ignore_write = false;
 
         $this->vars = null;
         $this->key  = session_id();
 
+        $this->log("Session regenerate: $old_id -> {$this->key}");
+
         return true;
     }
 
+    /**
+     * See if we have vars of this key already cached, and if so, return them.
+     *
+     * @param string $key Session identifier
+     *
+     * @return string Serialized data string
+     */
+    protected function get_cache($key)
+    {
+        // no session data in cache (read() returns false)
+        if (!$this->key) {
+            $cache = null;
+        }
+        // use internal data for fast requests (up to 0.5 sec.)
+        else if ($key == $this->key && (!$this->vars || microtime(true) - $this->start < 0.5)) {
+            $cache = $this->vars;
+        }
+        else { // else read data again
+            $cache = $this->read($key);
+        }
+
+        return $cache;
+    }
 
     /**
      * Append the given value to the certain node in the session data array
      *
-     * @param string Path denoting the session variable where to append the value
-     * @param string Key name under which to append the new value (use null for appending to an indexed list)
-     * @param mixed  Value to append to the session data array
+     * Warning: Do not use if you already modified $_SESSION in the same request (#1490608)
+     *
+     * @param string $path  Path denoting the session variable where to append the value
+     * @param string $key   Key name under which to append the new value (use null for appending to an indexed list)
+     * @param mixed  $value Value to append to the session data array
      */
     public function append($path, $key, $value)
     {
@@ -471,18 +361,20 @@ class rcube_session
         $this->appends[] = $path;
 
         // when overwriting a previously unset variable
-        if ($this->unsets[$path])
+        if ($this->unsets[$path]) {
             unset($this->unsets[$path]);
+        }
     }
 
-
     /**
      * Unset a session variable
      *
-     * @param string Variable name (can be a path denoting a certain node in the session array, e.g. compose.attachments.5)
-     * @return boolean True on success
+     * @param string $var Variable name (can be a path denoting a certain node
+     *                    in the session array, e.g. compose.attachments.5)
+     *
+     * @return boolean True on success, False on failure
      */
-    public function remove($var=null)
+    public function remove($var = null)
     {
         if (empty($var)) {
             return $this->destroy(session_id());
@@ -503,19 +395,20 @@ class rcube_session
         return true;
     }
 
-
     /**
      * Kill this session
      */
     public function kill()
     {
+        $this->log("Session destroy: " . session_id());
+
         $this->vars = null;
-        $this->ip = rcube_utils::remote_addr(); // update IP (might have changed)
+        $this->ip   = rcube_utils::remote_addr(); // update IP (might have changed)
         $this->destroy(session_id());
+
         rcube_utils::setcookie($this->cookiename, '-del-', time() - 60);
     }
 
-
     /**
      * Re-read session data from storage backend
      */
@@ -531,10 +424,9 @@ class rcube_session
             $node[$k] = $value;
         }
 
-        if ($this->key && $this->memcache)
-            $data = $this->mc_read($this->key);
-        else if ($this->key)
-            $data = $this->db_read($this->key);
+        if ($this->key) {
+            $data = $this->read($this->key);
+        }
 
         if ($data) {
             session_decode($data);
@@ -554,14 +446,13 @@ class rcube_session
                 }
             }
         }
-
     }
 
     /**
      * Returns a reference to the node in data array referenced by the given path.
      * e.g. ['compose','attachments'] will return $_SESSION['compose']['attachments']
      */
-    private function &get_node($path, &$data_arr)
+    protected function &get_node($path, &$data_arr)
     {
         $node = &$data_arr;
         if (!empty($path)) {
@@ -578,7 +469,7 @@ class rcube_session
     /**
      * Serialize session data
      */
-    private function serialize($vars)
+    protected function serialize($vars)
     {
         $data = '';
         if (is_array($vars)) {
@@ -592,12 +483,15 @@ class rcube_session
         return $data;
     }
 
-
     /**
      * Unserialize session data
      * http://www.php.net/manual/en/function.session-decode.php#56106
+     *
+     * @param string $str Serialized data string
+     *
+     * @return array Unserialized data
      */
-    private function unserialize($str)
+    public static function unserialize($str)
     {
         $str    = (string)$str;
         $endptr = strlen($str);
@@ -688,12 +582,13 @@ class rcube_session
             $p = $q;
         }
 
-        return unserialize( 'a:' . $items . ':{' . $serialized . '}' );
+        return unserialize('a:' . $items . ':{' . $serialized . '}');
     }
 
-
     /**
      * Setter for session lifetime
+     *
+     * @param int $lifetime Session lifetime (in seconds)
      */
     public function set_lifetime($lifetime)
     {
@@ -704,45 +599,58 @@ class rcube_session
         $this->now = $now - ($now % ($this->lifetime / 2));
     }
 
-
     /**
      * Getter for remote IP saved with this session
+     *
+     * @return string Client IP address
      */
     public function get_ip()
     {
         return $this->ip;
     }
 
-
     /**
      * Setter for cookie encryption secret
+     *
+     * @param string $secret Authentication secret string
      */
-    function set_secret($secret)
+    function set_secret($secret = null)
     {
-        $this->secret = $secret;
-    }
+        // generate random hash and store in session
+        if (!$secret) {
+            if (!empty($_SESSION['auth_secret'])) {
+                $secret = $_SESSION['auth_secret'];
+            }
+            else {
+                $secret = rcube_utils::random_bytes(strlen($this->key));
+            }
+        }
 
+        $_SESSION['auth_secret'] = $secret;
+    }
 
     /**
      * Enable/disable IP check
+     *
+     * @param bool $check IP address checking state
      */
     function set_ip_check($check)
     {
         $this->ip_check = $check;
     }
 
-
     /**
      * Setter for the cookie name used for session cookie
+     *
+     * @param string $name Authentication cookie name
      */
-    function set_cookiename($cookiename)
+    function set_cookiename($name)
     {
-        if ($cookiename) {
-            $this->cookiename = $cookiename;
+        if ($name) {
+            $this->cookiename = $name;
         }
     }
 
-
     /**
      * Check session authentication cookie
      *
@@ -780,31 +688,36 @@ class rcube_session
         return $result;
     }
 
-
     /**
      * Set session authentication cookie
      */
-    function set_auth_cookie()
+    public function set_auth_cookie()
     {
         $this->cookie = $this->_mkcookie($this->now);
         rcube_utils::setcookie($this->cookiename, $this->cookie, 0);
         $_COOKIE[$this->cookiename] = $this->cookie;
     }
 
-
     /**
-     * Create session cookie from session data
+     * Create session cookie for specified time slot.
      *
-     * @param int Time slot to use
+     * @param int $timeslot Time slot to use
+     *
+     * @return string Cookie value
      */
-    function _mkcookie($timeslot)
+    protected function _mkcookie($timeslot)
     {
-        $auth_string = "$this->key,$this->secret,$timeslot";
-        return "S" . (function_exists('sha1') ? sha1($auth_string) : md5($auth_string));
+        // make sure the secret key exists
+        $this->set_secret();
+
+        // no need to hash this, it's just a random string
+        return $_SESSION['auth_secret'] . '-' . $timeslot;
     }
 
     /**
      * Writes debug information to the log
+     *
+     * @param string Log line
      */
     function log($line)
     {
